/* packet-bmactest.c
 * Routines for PROTONAME dissection
 * Copyright 2000, Tyler McHenry <smchenry@isi.edu>
 *
 * $Id: packet-bmactest.c 4754 2006-01-31 23:58:47Z smchenry $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <epan/packet.h>

#include "packet-bmactest.h"

/* Forward declaration we need below */
void proto_reg_handoff_bmactest(void);

/* Initialize the protocol and registered fields */
static int proto_bmactest = -1;
static int hf_bmactest_source = -1;
static int hf_bmactest_seq_num = -1;
static int hf_bmactest_outstanding = -1;
static int hf_bmactest_lost = -1;
static int hf_bmactest_completed = -1;
static int hf_bmactest_comp_reported_start = -1;
static int hf_bmactest_comp_reported_cur = -1;

/* Initialize the subtree pointers */
static gint ett_bmactest = -1;

/* Code to actually dissect the packets */
void
dissect_bmactest(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
  proto_item *ti;
  proto_tree *bmactest_tree;

  if (check_col(pinfo->cinfo, COL_INFO)) 
    col_set_str(pinfo->cinfo, COL_INFO, "B-MAC Test Packet");

  printf("In bmactest: pinfo = %X\n", pinfo);

  SET_ADDRESS(&pinfo->src, AT_MOTE, 2, tvb_get_ptr(tvb, 0, 2));
  SET_ADDRESS(&pinfo->dl_src, AT_MOTE, 2, tvb_get_ptr(tvb, 0, 2));

  if (tree) {

    ti = proto_tree_add_item(tree, proto_bmactest, tvb, 0, -1, FALSE);

    bmactest_tree = proto_item_add_subtree(ti, ett_bmactest);

    proto_tree_add_item(bmactest_tree,
			hf_bmactest_source, tvb, 0, 2, TRUE);
    proto_tree_add_item(bmactest_tree,
			hf_bmactest_seq_num, tvb, 2, 4, TRUE);
    proto_tree_add_item(bmactest_tree,
			hf_bmactest_outstanding, tvb, 6, 1, TRUE);
    proto_tree_add_item(bmactest_tree,
			hf_bmactest_lost, tvb, 7, 2, TRUE);
    proto_tree_add_item(bmactest_tree,
			hf_bmactest_completed, tvb, 9, 4, TRUE);
    proto_tree_add_item(bmactest_tree,
			hf_bmactest_comp_reported_start, tvb, 13, 4, TRUE);
    proto_tree_add_item(bmactest_tree,
			hf_bmactest_comp_reported_cur, tvb, 17, 4, TRUE);
	
  }

}


/* Register the protocol with Ethereal */

void
proto_register_bmactest(void)
{                

/* Setup list of header fields  See Section 1.6.1 for details*/
	static hf_register_info hf[] = {
	  { &hf_bmactest_source,
	    { "Source Address",           "bmactest.source",
	      FT_UINT16, BASE_DEC, NULL, 0,          
	      "Sender of this packet", HFILL }
	  },
	  { &hf_bmactest_seq_num,
	    { "Sequence Number",           "bmactest.seq_num",
	      FT_UINT32, BASE_DEC, NULL, 0,          
	      "Unique identifier for this packet", HFILL }
	  },
	  { &hf_bmactest_outstanding,
	    { "Packets Outstanding",           "bmactest.outstanding",
	      FT_UINT8, BASE_DEC, NULL, 0,          
	      "How many packets am I still expecting to hear back?", HFILL }
	  },
	  { &hf_bmactest_lost,
	    { "Packets Lost",           "bmactest.lost",
	      FT_UINT16, BASE_DEC, NULL, 0,          
	      "How many packets have I given up on hearing?", HFILL }
	  },
	  { &hf_bmactest_completed,
	    { "Packets Completed",           "bmactest.completed",
	      FT_UINT32, BASE_DEC, NULL, 0,          
	      "Number of packets that have made a full loop", HFILL }
	  },
	  { &hf_bmactest_comp_reported_start,
	    { "Completions Reported (at start of this packet)",           "bmactest.comp_reported_start",
	      FT_UINT32, BASE_DEC, NULL, 0,          
	      "How many broadcasts Node 1 had sent before starting this packet", HFILL }
	  },
	  { &hf_bmactest_comp_reported_cur,
	    { "Completions Reported (as of this hop)",           "bmactest.comp_reported_cur",
	      FT_UINT32, BASE_DEC, NULL, 0,          
	      "How many broadcasts have been heard from Node 1 in all", HFILL }
	  },
	};

/* Setup protocol subtree array */
	static gint *ett[] = {
		&ett_bmactest,
	};

/* Register the protocol name and description */
	proto_bmactest = proto_register_protocol("B-MAC Test Application",
	    "BMACTest", "bmactest");

/* Required function calls to register the header fields and subtrees used */
	proto_register_field_array(proto_bmactest, hf, array_length(hf));
	proto_register_subtree_array(ett, array_length(ett));
        
}


void
proto_reg_handoff_bmactest(void)
{
  register_dissector("bmactest", dissect_bmactest, proto_bmactest);
}


