/* packet-rssi.c
 * Routines for PROTONAME dissection
 * Copyright 2000, Tyler McHenry <smchenry@isi.edu>
 *
 * $Id: packet-rssi.c 5030 2006-04-02 07:37:43Z smchenry $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include <epan/packet.h>
#include "prefs.h"

#include "packet-moteradio.h"
#include "packet-rssi.h"


/* Values for the "type" field. */

/* Forward declaration we need below */
void proto_reg_handoff_rssi(void);

/* Initialize the protocol and registered fields */
int proto_rssi = -1;
static int hf_rssi_signal_strength = -1;

/* Initialize the subtree pointers */
static gint ett_rssi = -1;

/* Preferences */

/* Code to actually dissect the packets */
void
dissect_rssi(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{

  proto_item *ti;
  proto_tree *rssi_tree;

  if (check_col(pinfo->cinfo, COL_INFO)) 
    col_set_str(pinfo->cinfo, COL_INFO, "RSSI Measurement");
    
  if (tree) {

    ti = proto_tree_add_item(tree, proto_rssi, tvb, 0, -1, FALSE);

    rssi_tree = proto_item_add_subtree(ti, ett_rssi);

    proto_tree_add_item(rssi_tree,
			hf_rssi_signal_strength, tvb, 0, 2, TRUE);
  }
}


/* Register the protocol with Ethereal */

void
proto_register_rssi(void)
{                
  module_t *rssi_module;
 

/* Setup list of header fields  See Section 1.6.1 for details*/
	static hf_register_info hf[] = {
	  { &hf_rssi_signal_strength,
	    { "Signal Strength",           "rssi.signal_strength",
	      FT_UINT16, BASE_DEC, NULL, 0,          
	      "Magnitude of Signal Strength is inversely proportional to the strength of signal detected by the mote.", HFILL }
	  },
	};

/* Setup protocol subtree array */
	static gint *ett[] = {
		&ett_rssi,
	};

/* Register the protocol name and description */
	proto_rssi = proto_register_protocol("Received Signal Strength Indicator Measurement",
	    "RSSI", "rssi");

/* Required function calls to register the header fields and subtrees used */
	proto_register_field_array(proto_rssi, hf, array_length(hf));
	proto_register_subtree_array(ett, array_length(ett));
        
/* Preferences Module */


}


void
proto_reg_handoff_rssi(void)
{
  register_dissector("rssi", dissect_rssi, proto_rssi);
}


