/* packet-smactest.c
 * Routines for S-MAC Test Application dissection
 * Copyright 2005, Tyler McHenry <smchenry@isi.edu>
 *
 * $Id: packet-smactest.c 4312 2005-08-07 11:03:47Z smchenry $
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <math.h>
#include <glib.h>

#include <epan/packet.h>
#include <epan/address.h>
#include <epan/conversation.h>
#include <epan/tap.h>
#include "prefs.h"

#include "packet-smac.h"
#include "packet-smactest.h"
#include "packet-linkstate.h"

/* Forward declaration we need below */

/* External functions */
void proto_reg_handoff_smactest(void);
void smactest_init(void);

/* Initialize the protocol and registered fields */
int proto_smactest = -1;
static int hf_smactest_num_tx_bcast = -1;
static int hf_smactest_num_tx_ucast = -1;

/* Preferences */

/* Initialize the subtree pointers */
static gint ett_smactest = -1;

static dissector_handle_t linkstate_handle;

/* Code to actually dissect the packets */

static void
dissect_smactest(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
  tvbuff_t *next_tvb;
  proto_item *ti;
  proto_tree *smactest_tree;
  guint16 length = tvb_length(tvb);
  guint i;

  if (check_col(pinfo->cinfo, COL_INFO)) 
    col_set_str(pinfo->cinfo, COL_INFO, "S-MAC Test Packet");

  if (tree) {
    ti = proto_tree_add_item(tree, proto_smactest, tvb, 0, -1, FALSE);
    smactest_tree = proto_item_add_subtree(ti, ett_smactest);

    proto_tree_add_item(smactest_tree,
			hf_smactest_num_tx_bcast, tvb, 0, 1, TRUE);
    proto_tree_add_item(smactest_tree,
			hf_smactest_num_tx_ucast, tvb, 1, 1, TRUE);
  }

  linkstate_handle = find_dissector("linkstate");    

  for (i = SMACTEST_INFO_LEN; (i + LINKSTATE_PACKET_LEN) <= length;
       i += LINKSTATE_PACKET_LEN) {

    next_tvb = tvb_new_subset(tvb, i, LINKSTATE_PACKET_LEN,
			      LINKSTATE_PACKET_LEN);
  
    call_dissector(linkstate_handle, next_tvb, pinfo, tree);
  }

}

/* Register the protocol with Ethereal */

void
proto_register_smactest(void)
{
  static gint *ett[] = {
    &ett_smactest,
  };

  static hf_register_info hf[] = {
    { &hf_smactest_num_tx_bcast,
      { "Broadcast Packets Transmitted",           "smactest.num_tx_bcast",
	FT_UINT8, BASE_DEC, NULL, 0,          
	"Broadcasts transmitted by this node", HFILL }
    },
    { &hf_smactest_num_tx_ucast,
      { "Unicast Packets Transmitted",           "smactest.num_tx_ucast",
	FT_UINT8, BASE_DEC, NULL, 0,          
	"Unicasts transmitted by this node ", HFILL }
    },
  };

  register_init_routine(&smactest_init);

  proto_smactest = proto_register_protocol("S-MAC Test Application",
					   "SMACTest", "smactest");

  proto_register_field_array(proto_smactest, hf, array_length(hf));
  proto_register_subtree_array(ett, array_length(ett));
}

void
smactest_init(void)
{
  /* n/t */
}

void
proto_reg_handoff_smactest(void)
{
  linkstate_handle = find_dissector("linkstate");

  register_dissector("smactest", dissect_smactest, proto_smactest);

  register_init_routine(&smactest_init);
}


