/* smac_schedules.h
 * S-MAC schedules addition for ethereal
 *
 * $Id: smac_schedules.h 5883 2006-10-30 19:36:22Z smchenry $
 *
 * Copyright 2005, University of Southern California
 * By Tyler McHenry <smchenry@isi.edu>
 *
 * based on voip_calls.h
 * Copyright 2004, Ericsson , Spain
 * By Francisco Alcoba <francisco.alcoba@ericsson.com>
 *
 * based on h323_calls.h
 * Copyright 2004, Iskratel, Ltd, Kranj
 * By Miha Jemec <m.jemec@iskratel.si>
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation,  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#ifndef SMAC_SCHEDULES_H_INCLUDED
#define SMAC_SCHEDULES_H_INCLUDED

#include <glib.h>
#include <stdio.h>
#include <epan/packet.h>
#include <epan/address.h>

#include <epan/dissectors/packet-moteradio.h>
#include <epan/dissectors/packet-smac.h>
#include "graph_analysis.h"

/****************************************************************************/

/*** S-MAC constants have been moved to the dissector header file! ***/

typedef struct _smac_collisions {
    int num_total;
    int num_sync;
    int num_rcb;
    int num_data;
} smac_collisions;

/* defines a smac schedule */
typedef struct _smac_schedule_info {
    guint16  id;
    guint16  total_nodes;
    guint16  active_nodes;
    guint16  origin_node;
    guint16  packets;
    guint16  bytes;
    guint16  periods;
    guint16  sync_packets;
    guint16  first_sync_s;
    guint16  first_sync_us;
    guint16  last_sync_s;
    guint16  last_sync_us;
    guint64  phase;
    GList*   nodes_list;
    GList*   broadcast_packets_list;
    smac_collisions collisions;
    gboolean selected;
} smac_schedule_info_t;

typedef struct _smac_schedule_nodestatus {
    smac_schedule_info_t* schedule;
    guint64 last_sync;
    gboolean active;
} smac_schedule_nodestatus_t;

typedef struct _smac_node_info {
    address addr;
    GList* schedules_list; 
    GList* status_list; 
} smac_node_info_t;

typedef struct _smac_estimate_t {
    gdouble exact_value;
    guint64 value;
    guint64 error;
    gdouble correlation;
} smac_estimate_t;

typedef struct _smac_loose_packet_t {
    packet_info pinfo; /* THIS IS A COPY!! */
    mote_priv_data *m_priv;
    smac_priv_data *s_priv;
} smac_loose_packet_t;

/* structure that holds the information about all detected schedules */
/* struct holding all information of the tap */

typedef struct _smac_schedules_tapinfo {
    int     num_packets;
    int     num_sync_packets;
    guint64 time_zero;
    guint64 time_now;
    GList*  schedules_list;   /* list with all schedules */
    GList*  nodes_list;   /* list with all nodes */
    GList*  loose_packets_list;
    smac_collisions collisions;
    GList*  collision_packets_list;
    int     dummy;
} smac_schedules_tapinfo_t;

#define EMPTY_COLLISIONS_STRUCT {0,0,0,0}
#define EMPTY_TAPINFO_STRUCT {0,0,0,0,NULL,NULL,NULL,EMPTY_COLLISIONS_STRUCT, NULL, 0}
#define NODE_ID(n) (*((guint16*)((n)->addr.data)))

/****************************************************************************/
/* INTERFACE */

extern int smac_vis_tap;
extern guint64 smac_measured_period;

/*
 * Retrieves a constant reference to the unique info structure of the smac_schedules tap listener.
 * The user should not modify the data pointed to.
 */
extern const smac_schedules_tapinfo_t* smac_schedules_get_info(void);

/*
 * Cleans up memory of smac schedules tap.
 */
extern void smac_schedules_reset(smac_schedules_tapinfo_t *tapinfo);

extern void smac_schedules_remove();

extern void graph_analysis_data_init(void);

extern int smac_schedules_packet(void *t, packet_info *pinfo, epan_dissect_t *edt, void *priv);

extern void smac_schedules_reanalyze_loose_packets();

extern void smac_schedules_check_inactive();

extern gboolean smac_addr_is_broadcast(address* addr);

#endif /*SMAC_SCHEDULES_H_INCLUDED*/
