/* smac_schedules_dlg.c
 * S-MAC schedules addition for ethereal
 *
 * $Id: smac_schedules_dlg.c 4729 2006-01-25 00:58:49Z smchenry $
 *
 * Copyright 2005, University of Southern California
 * By Tyler McHenry <smchenry@isi.edu>
 *
 * based on voip_conversations_dlg.c
 * Copyright 2004, Ericsson , Spain
 * By Francisco Alcoba <francisco.alcoba@ericsson.com>
 *
 * based on h323_conversations_dlg.c
 * Copyright 2004, Iskratel, Ltd, Kranj
 * By Miha Jemec <m.jemec@iskratel.si>
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation,  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <ctype.h>

#include "register.h"

#include "smac_schedules.h"
#include "smac_schedules_dlg.h"
#include "smac_visualization.h"
#include "smac_visualization_dlg.h" 

#include "globals.h"
#include "epan/filesystem.h"

#include <epan/tap.h>
#include "tap_menu.h"
#include "dlg_utils.h"
#include "ui_util.h"
#include "compat_macros.h"
#include "gtkglobals.h"

#include "image/clist_ascend.xpm"
#include "image/clist_descend.xpm"
#include "simple_dialog.h"

#include <epan/to_str.h>

#include <string.h>
#include <ctype.h>
#include <epan/address.h>
#include <epan/addr_resolv.h>

static const gchar FWD_LABEL_TEXT[] = "Select one schedule.";

/****************************************************************************/
/* pointer to the one and only dialog window */
static GtkWidget *smac_schedules_dlg = NULL;
static smac_visualization_dlg_t *smac_visualization_dlg = NULL;

static GtkWidget *clist = NULL;
static GtkWidget *ctree = NULL;
static GtkWidget *top_label = NULL;
static GtkWidget *status_label = NULL;
static GtkWidget *label_fwd = NULL;

static GtkWidget *bt_unselect = NULL;
static GtkWidget *bt_filter = NULL;
static GtkWidget *bt_visualization = NULL;

static smac_schedule_info_t* selected_schedule_fwd = NULL;  /* current selection */
static GList *last_list = NULL;

static guint32 schedules_nb = 0;     /* number of displayed schedules */
static guint32 schedules_ns = 0;     /* number of selected schedules */

/* static smac_visualization_data_t *smac_visualization_data; */

#define NUM_COLS 10

/****************************************************************************/

/* Append a line to clist */
static void add_to_clist(smac_schedule_info_t* schedule)
{
    gint added_row;
    gchar *data[NUM_COLS];
    gchar field[NUM_COLS][50];
    gint c;

    for (c=0;c<NUM_COLS;c++){
        data[c]=&field[c][0];
    }
  
    g_snprintf(field[0], 15, "%d", schedule->id);
    g_snprintf(field[1], 15, "%llu", schedule->phase);
    g_snprintf(field[2], 15, "%d", schedule->total_nodes);
    g_snprintf(field[3], 15, "%d", schedule->active_nodes);
    g_snprintf(field[4], 15, "%d", schedule->origin_node);
    g_snprintf(field[5], 15, "%d", schedule->packets);
    g_snprintf(field[6], 15, "%d", schedule->bytes);
    g_snprintf(field[7], 15, "%d", schedule->periods);
    g_snprintf(field[8], 15, "%d.%.6d", schedule->first_sync_s, schedule->first_sync_us);
    g_snprintf(field[9], 15, "%d.%.6d", schedule->last_sync_s, schedule->last_sync_us);

    added_row = gtk_clist_append(GTK_CLIST(clist), data);

    gtk_clist_set_row_data(GTK_CLIST(clist), added_row, schedule);

}


void smac_schedules_remove_tap_listener(void)
{
    smac_schedules_remove();
}

/****************************************************************************/
/* CALLBACKS                                                                */
/****************************************************************************/
static void
smac_schedules_on_destroy                      (GtkObject       *object _U_,
                                                gpointer         user_data _U_)
{
    /* remove_tap_listeners */
    smac_schedules_remove_tap_listener();

    /* Clean up memory used by schedules tap */
    smac_schedules_reset((smac_schedules_tapinfo_t*) smac_schedules_get_info());

    /* Note that we no longer have a "Smac Schedules" dialog box. */
    smac_schedules_dlg = NULL;
}


/****************************************************************************/
static void
smac_schedules_on_unselect                  (GtkButton       *button _U_,
                                             gpointer         user_data _U_)
{
    gtk_clist_unselect_all(GTK_CLIST(clist));
    gtk_label_set_text(GTK_LABEL(label_fwd), FWD_LABEL_TEXT);

    gtk_widget_set_sensitive(bt_unselect, FALSE);
    gtk_widget_set_sensitive(bt_filter, FALSE);
    gtk_widget_set_sensitive(bt_visualization, TRUE);
}


/****************************************************************************/
static void
smac_schedules_on_filter                    (GtkButton       *button _U_,
                                             gpointer         user_data _U_)
{
    const gchar *filter_string;
    gchar c;
    GString *filter_string_fwd;
    GString *filter_string_fwd_src;
    GString *filter_string_fwd_dst;
    GString *filter_string_fwd_bcast;
    gchar *filter_prepend;
    gboolean isFirst = TRUE;
    GList *schedules_list;
    GList *nodes_list;
    GList *bcast_list;
    guint filter_length = 0;
    smac_schedule_info_t *schedule = NULL;
    smac_node_info_t *node = NULL;
    guint32 *frame_number = NULL;
  
    if (selected_schedule_fwd == NULL) return;

    filter_string=gtk_entry_get_text(GTK_ENTRY(main_display_filter_widget));
    filter_length = strlen(filter_string);
    filter_prepend = "";
    while ((c = *filter_string++) != '\0') {
        if (!isspace((guchar)c)) {
            /* The filter string isn't blank, so there's already
               an expression; we OR in the new expression */
            filter_prepend = " or ";
            break;
        }
    }
		
    filter_string_fwd_src = g_string_new(NULL);
    filter_string_fwd_dst = g_string_new(NULL);
    filter_string_fwd_bcast = g_string_new(NULL);
    filter_string_fwd = g_string_new(filter_prepend);

    for (schedules_list = g_list_first(smac_schedules_get_info()->schedules_list);
         schedules_list; schedules_list = g_list_next(schedules_list)) {
    
        schedule = schedules_list->data;
        if (schedule->id == selected_schedule_fwd->id) break;   
        else schedule = NULL;
    }

    if (schedule == NULL) goto free_strings_and_return;

    g_string_sprintfa(filter_string_fwd_src, "((smac.type >= 1 || (smac.type == 0 && smac.dest != %u)) && (", MOTERADIO_BROADCAST);
    g_string_sprintfa(filter_string_fwd_dst, "((smac.type < 2) && (");
    g_string_sprintfa(filter_string_fwd_bcast, "(");
  
    for (nodes_list = g_list_first(schedule->nodes_list);
         nodes_list; nodes_list = g_list_next(nodes_list)) {
    
        node = nodes_list->data;

        g_string_sprintfa(filter_string_fwd_src, "%ssmac.source == %d", isFirst ? "" : " || ", 
                          NODE_ID(node));
        g_string_sprintfa(filter_string_fwd_dst, "%ssmac.dest == %d", isFirst ? "" : " || ",
                          NODE_ID(node));
        isFirst = FALSE;
    }

    isFirst = TRUE;

    for (bcast_list = g_list_first(schedule->broadcast_packets_list); bcast_list;
         bcast_list = g_list_next(bcast_list)) {

        frame_number = bcast_list->data;

        g_string_sprintfa(filter_string_fwd_bcast, "%sframe.number == %d", isFirst ? "" : " || ",
                          *frame_number); 
        isFirst = FALSE;
    }

    g_string_sprintfa(filter_string_fwd_src, ")) || ");
    g_string_sprintfa(filter_string_fwd_dst, ")) || ");
    g_string_sprintfa(filter_string_fwd_bcast, ") ");
    filter_length = filter_length + filter_string_fwd_src->len + 
        filter_string_fwd_dst->len + filter_string_fwd_bcast->len; 

    g_string_sprintfa(filter_string_fwd, 
                      "((frame.time_relative >= %d.%.6d && frame.time_relative <= %d.%.6d) && %s%s%s)", 
                      schedule->first_sync_s, schedule->first_sync_us, 
                      schedule->last_sync_s, schedule->last_sync_us,
                      filter_string_fwd_src->str, filter_string_fwd_dst->str,
                      filter_string_fwd_bcast->str);
  
    gtk_entry_append_text(GTK_ENTRY(main_display_filter_widget), filter_string_fwd->str);

  free_strings_and_return:
    g_string_free(filter_string_fwd_src, TRUE);
    g_string_free(filter_string_fwd_dst, TRUE);
    g_string_free(filter_string_fwd, TRUE);
  
}




/****************************************************************************/
static void
on_visualization_bt_clicked                    (GtkButton       *button _U_,
                                                gpointer         user_data _U_)
{
    smac_schedules_tapinfo_t* tapinfo = ((smac_schedules_tapinfo_t*) smac_schedules_get_info());

    printf("viz_bt_clicked\n");
    
    /* smac_visualization_dump_all(); */
    smac_visualization_dlg = smac_visualization_dlg_open(smac_visualization_dlg);

    smac_visualization_dlg->params.schedules_list = tapinfo->schedules_list;
    smac_visualization_dlg->params.nodes_list = tapinfo->nodes_list;
    smac_visualization_dlg->params.node_streams = smac_node_streams;
    smac_visualization_dlg->params.start_time = tapinfo->time_zero;
    smac_visualization_dlg->params.duration = tapinfo->time_now - tapinfo->time_zero;

    smac_visualization_dlg_draw(smac_visualization_dlg);
}

static const GdkColor COLOR_SELECT = {0, 0x00ff, 0x80ff, 0x80ff};
static const GdkColor COLOR_DEFAULT = {0, 0xffff, 0xffff, 0xffff};

/****************************************************************************/
/* when the user selects a row in the schedules list */
static void
smac_schedules_on_select_row(GtkCList *clist,
                             gint row _U_,
                             gint column _U_,
                             GdkEventButton *event _U_,
                             gpointer user_data _U_)
{
    GdkColor color = COLOR_DEFAULT;
	
    selected_schedule_fwd = gtk_clist_get_row_data(GTK_CLIST(clist), row); 

    gtk_clist_set_background(GTK_CLIST(clist), row, &color);
	
    gtk_widget_set_sensitive(bt_unselect, TRUE);
    gtk_widget_set_sensitive(bt_filter, TRUE);
    gtk_widget_set_sensitive(bt_visualization, TRUE);

    /* TODO: activate other buttons when implemented */
}


/****************************************************************************/

typedef struct column_arrows {
    GtkWidget *table;
    GtkWidget *ascend_pm;
    GtkWidget *descend_pm;
} column_arrows;


/****************************************************************************/
static void
smac_schedules_click_column_cb(GtkCList *clist, gint column, gpointer data)
{
    column_arrows *col_arrows = (column_arrows *) data;
    int i;

    gtk_clist_freeze(clist);

    for (i=0; i<NUM_COLS; i++) {
        gtk_widget_hide(col_arrows[i].ascend_pm);
        gtk_widget_hide(col_arrows[i].descend_pm);
    }

    if (column == clist->sort_column) {
        if (clist->sort_type == GTK_SORT_ASCENDING) {
            clist->sort_type = GTK_SORT_DESCENDING;
            gtk_widget_show(col_arrows[column].descend_pm);
        } else {
            clist->sort_type = GTK_SORT_ASCENDING;
            gtk_widget_show(col_arrows[column].ascend_pm);
        }
    } else {
        clist->sort_type = GTK_SORT_ASCENDING;
        gtk_widget_show(col_arrows[column].ascend_pm);
        gtk_clist_set_sort_column(clist, column);
    }
    gtk_clist_thaw(clist);

    gtk_clist_sort(clist);
}


/****************************************************************************/
static gint
smac_schedules_sort_column(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
    char *text1 = NULL;
    char *text2 = NULL;
    int i1, i2;

    const GtkCListRow *row1 = (const GtkCListRow *) ptr1;
    const GtkCListRow *row2 = (const GtkCListRow *) ptr2;

    text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
    text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;

    /* Note: ALL values in this dialog are numeric */

    i1=atoi(text1);
    i2=atoi(text2);
    return i1-i2;
}


/****************************************************************************/
/* INTERFACE                                                                */
/****************************************************************************/

static void smac_schedules_dlg_initpage_nodes(GtkWidget *main_vb, GtkWidget *win)
{
    GtkWidget *scrolledwindow;
    GtkWidget *hbuttonbox;
    GtkWidget *bt_close;
    GtkTooltips *tooltips = gtk_tooltips_new();

    scrolledwindow = scrolled_window_new(NULL,NULL);
    gtk_box_pack_start(GTK_BOX(main_vb), scrolledwindow, TRUE, TRUE, 0);

    ctree = gtk_ctree_new(1,0);
    gtk_container_add(GTK_CONTAINER(scrolledwindow), ctree);

    hbuttonbox = gtk_hbutton_box_new ();
    bt_close = BUTTON_NEW_FROM_STOCK(GTK_STOCK_CLOSE);
    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_close);
    GTK_WIDGET_SET_FLAGS(bt_close, GTK_CAN_DEFAULT);
    gtk_tooltips_set_tip (tooltips, bt_close, "Close this dialog", NULL);

    window_set_cancel_button(win, bt_close, window_cancel_button_cb);  

}

static void smac_schedules_dlg_initpage_schedules(GtkWidget *main_vb, GtkWidget *win)
{
    GtkWidget *scrolledwindow;
    GtkWidget *hbuttonbox;
    GtkWidget *bt_close;
    GtkTooltips *tooltips = gtk_tooltips_new();
    gchar *titles[NUM_COLS] =  {"Schedule", "Phase", "Total Nodes", "Active Nodes", "Origin Node",  "Packets", "Bytes", "Periods", "Fist SYNC", "Last SYNC"};
    column_arrows *col_arrows;
    GtkWidget *column_lb;
    int i;

    top_label = gtk_label_new ("Detected 0 S-MAC Schedules. Selected 0 Schedules.");
    gtk_box_pack_start (GTK_BOX (main_vb), top_label, FALSE, FALSE, 8);

    scrolledwindow = scrolled_window_new (NULL, NULL);
    gtk_box_pack_start (GTK_BOX (main_vb), scrolledwindow, TRUE, TRUE, 0);

    clist = gtk_clist_new (NUM_COLS);
    gtk_container_add (GTK_CONTAINER (scrolledwindow), clist);

    gtk_clist_set_column_width (GTK_CLIST (clist), 0, 70);
    gtk_clist_set_column_width (GTK_CLIST (clist), 1, 60);
    gtk_clist_set_column_width (GTK_CLIST (clist), 2, 80);
    gtk_clist_set_column_width (GTK_CLIST (clist), 3, 85);
    gtk_clist_set_column_width (GTK_CLIST (clist), 4, 75);
    gtk_clist_set_column_width (GTK_CLIST (clist), 5, 60);
    gtk_clist_set_column_width (GTK_CLIST (clist), 6, 70);
    gtk_clist_set_column_width (GTK_CLIST (clist), 7, 60);
    gtk_clist_set_column_width (GTK_CLIST (clist), 8, 75);
    gtk_clist_set_column_width (GTK_CLIST (clist), 9, 75);

    gtk_clist_set_column_justification(GTK_CLIST(clist), 0, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(clist), 1, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(clist), 2, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(clist), 3, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(clist), 4, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(clist), 5, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(clist), 6, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(clist), 7, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(clist), 8, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(clist), 9, GTK_JUSTIFY_RIGHT);

    gtk_clist_column_titles_show (GTK_CLIST (clist));

    gtk_clist_set_compare_func(GTK_CLIST(clist), smac_schedules_sort_column);
    gtk_clist_set_sort_column(GTK_CLIST(clist), 0);
    gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_ASCENDING);

    /* sort by column feature */
    col_arrows = (column_arrows *) g_malloc(sizeof(column_arrows) * NUM_COLS);

    for (i=0; i<NUM_COLS; i++) {
        col_arrows[i].table = gtk_table_new(2, 2, FALSE);
        gtk_table_set_col_spacings(GTK_TABLE(col_arrows[i].table), 5);
        column_lb = gtk_label_new(titles[i]);
        gtk_table_attach(GTK_TABLE(col_arrows[i].table), column_lb, 0, 1, 0, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
        gtk_widget_show(column_lb);

        col_arrows[i].ascend_pm = xpm_to_widget(clist_ascend_xpm);
        gtk_table_attach(GTK_TABLE(col_arrows[i].table), col_arrows[i].ascend_pm, 1, 2, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
        col_arrows[i].descend_pm = xpm_to_widget(clist_descend_xpm);
        gtk_table_attach(GTK_TABLE(col_arrows[i].table), col_arrows[i].descend_pm, 1, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
        /* make start time be the default sort order */
        if (i == 0) {
            gtk_widget_show(col_arrows[i].ascend_pm);
        }
        gtk_clist_set_column_widget(GTK_CLIST(clist), i, col_arrows[i].table);
        gtk_widget_show(col_arrows[i].table);
    }

    SIGNAL_CONNECT(clist, "click-column", smac_schedules_click_column_cb, col_arrows);

    label_fwd = gtk_label_new (FWD_LABEL_TEXT);
    gtk_box_pack_start (GTK_BOX (main_vb), label_fwd, FALSE, FALSE, 0);

    status_label = gtk_label_new ("Total: Nodes: 0   Schedules: 0   Sync Packets: 0   Packets with no known schedule: 0");
    gtk_box_pack_start (GTK_BOX (main_vb), status_label, FALSE, FALSE, 8);

    /* button row */
    hbuttonbox = gtk_hbutton_box_new ();
    gtk_box_pack_start (GTK_BOX (main_vb), hbuttonbox, FALSE, FALSE, 0);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_SPREAD);
    gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox), 30);

    /*bt_unselect = gtk_button_new_with_label ("Unselect");
      gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_unselect);
      gtk_tooltips_set_tip (tooltips, bt_unselect, "Unselect this schedule", NULL);*/

    bt_filter = gtk_button_new_with_label ("Prepare Filter");
    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_filter);
    gtk_tooltips_set_tip (tooltips, bt_filter, "Prepare a display filter of the selected schedule", NULL);

    bt_visualization = gtk_button_new_with_label("Visualization");
    gtk_container_add(GTK_CONTAINER(hbuttonbox), bt_visualization);
    gtk_widget_show(bt_visualization);
    SIGNAL_CONNECT(bt_visualization, "clicked", on_visualization_bt_clicked, NULL);
    gtk_tooltips_set_tip (tooltips, bt_visualization, "Show a visualization of the radio state of the selected nodes over time.", NULL);

    bt_close = BUTTON_NEW_FROM_STOCK(GTK_STOCK_CLOSE);
    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_close);
    GTK_WIDGET_SET_FLAGS(bt_close, GTK_CAN_DEFAULT);
    gtk_tooltips_set_tip (tooltips, bt_close, "Close this dialog", NULL);

    SIGNAL_CONNECT(clist, "select_row", smac_schedules_on_select_row, NULL);
    /*SIGNAL_CONNECT(bt_unselect, "clicked", smac_schedules_on_unselect, NULL);*/
    SIGNAL_CONNECT(bt_filter, "clicked", smac_schedules_on_filter, NULL);

    window_set_cancel_button(win, bt_close, window_cancel_button_cb);
}

static void smac_schedules_dlg_create (void)
{
    GtkWidget *smac_schedules_dlg_w;
    GtkWidget *main_vb;
    GtkWidget *inner_vb_s;
    GtkWidget *inner_vb_n;
    GtkWidget *page_lb;
    GtkWidget *nb; /* Notebook */

    ((smac_schedules_tapinfo_t*) smac_schedules_get_info())->schedules_list = NULL; 

    smac_schedules_dlg_w=window_new(GTK_WINDOW_TOPLEVEL, "Ethereal: S-MAC Schedules");

    gtk_window_set_default_size(GTK_WINDOW(smac_schedules_dlg_w), 840, 350);

    main_vb = gtk_vbox_new (FALSE, 0);
    gtk_container_add(GTK_CONTAINER(smac_schedules_dlg_w), main_vb);
    gtk_container_set_border_width (GTK_CONTAINER (main_vb), 12);

    nb = gtk_notebook_new();
    gtk_container_add(GTK_CONTAINER(main_vb), nb);

    page_lb = gtk_label_new("By Schedule");
    inner_vb_s = gtk_vbox_new(FALSE,6);
    gtk_container_set_border_width(GTK_CONTAINER(inner_vb_s), 6);
    smac_schedules_dlg_initpage_schedules(inner_vb_s, smac_schedules_dlg_w);
    gtk_notebook_append_page(GTK_NOTEBOOK(nb), inner_vb_s, page_lb);
    gtk_widget_show(smac_schedules_dlg_w);

    page_lb = gtk_label_new("By Node");
    inner_vb_n = gtk_vbox_new(FALSE,6);
    gtk_container_set_border_width(GTK_CONTAINER(inner_vb_n), 6);
    smac_schedules_dlg_initpage_nodes(inner_vb_n, smac_schedules_dlg_w);
    gtk_notebook_append_page(GTK_NOTEBOOK(nb), inner_vb_n, page_lb);
    gtk_widget_show(smac_schedules_dlg_w);

    SIGNAL_CONNECT(smac_schedules_dlg_w, "delete_event", window_delete_event_cb, NULL);
    SIGNAL_CONNECT(smac_schedules_dlg_w, "destroy", smac_schedules_on_destroy, NULL);

    gtk_widget_show_all(smac_schedules_dlg_w);
    window_present(smac_schedules_dlg_w);

    smac_schedules_on_unselect(NULL, NULL);

    smac_schedules_dlg = smac_schedules_dlg_w;
}


/****************************************************************************/
/* PUBLIC								    */
/****************************************************************************/

/****************************************************************************/
/* update the contents of the dialog box clist */
/* list: pointer to list of smac_schedule_info_t* */

gint smac_schedules_dlg_compare_nodes(gconstpointer node_a, gconstpointer node_b) 
{
    const smac_node_info_t* node_info_a;
    const smac_node_info_t* node_info_b;
    gint node_num_a;
    gint node_num_b;

    node_info_a = (smac_node_info_t*)node_a;
    node_info_b = (smac_node_info_t*)node_b;

    node_num_a = NODE_ID(node_info_a);
    node_num_b = NODE_ID(node_info_b);

    if (node_num_a == node_num_b) return 0;
    else return (node_num_a < node_num_b) ? -1 : 1;
}

gint smac_schedules_dlg_compare_schedules(gconstpointer node_a, gconstpointer node_b) 
{
    if (((smac_schedule_info_t*)node_a)->id == ((smac_schedule_info_t*)node_b)->id)
        return 0;
    else
        return (((smac_schedule_info_t*)node_a)->id < ((smac_schedule_info_t*)node_b)->id) ? -1 : 1;
}

GtkCTreeNode* smac_schedules_dlg_update_nodes_add_node(GtkCTreeNode* parent, 
                                                       smac_node_info_t* new_node,
                                                       gboolean expand)
{
    gboolean leaf = !expand;
    gint node_num;
    gchar* name_buf;

    node_num = NODE_ID(new_node);
    name_buf = g_strdup_printf("Node %d", node_num);

    return gtk_ctree_insert_node(GTK_CTREE(ctree), GTK_CTREE_NODE(parent),
                                 GTK_CTREE_NODE(NULL), &name_buf,
                                 0, NULL, NULL, NULL, NULL, leaf, expand); 
  
}

GtkCTreeNode* smac_schedules_dlg_update_nodes_add_schedule(GtkCTreeNode* parent, 
							   smac_schedule_info_t* 
							   new_schedule)
{
    gchar *name_buf;

    name_buf = g_strdup_printf("Schedule %d", new_schedule->id);

    return gtk_ctree_insert_node(GTK_CTREE(ctree), GTK_CTREE_NODE(parent),
                                 GTK_CTREE_NODE(NULL), &name_buf,
                                 0, NULL, NULL, NULL, NULL, FALSE, FALSE); 
}

void smac_schedules_dlg_update_nodes(GList *list)
{
    GtkCTreeNode* node_root = NULL;
    GtkCTreeNode* cur_node_tn = NULL;
    GtkCTreeNode* cur_schedule_tn = NULL;
    smac_node_info_t* cur_node = NULL;
    smac_schedule_info_t* cur_schedule = NULL;
    GList* node_list = NULL;
    GList* tmp_list = NULL;
    GList* tmp_list2 = NULL;
    gchar* name_buf;

    if (smac_schedules_dlg == NULL) return;
    gtk_clist_clear(GTK_CLIST(ctree)); 

    /*  smac_schedules_reanalyze_loose_packets(); */
    smac_schedules_check_inactive();

    name_buf = g_strdup_printf("All Nodes");

    node_root = gtk_ctree_insert_node(GTK_CTREE(ctree), GTK_CTREE_NODE(NULL),
                                      GTK_CTREE(NULL), &name_buf, 0, 
                                      NULL, NULL, NULL, NULL, FALSE, TRUE);

    node_list = g_list_sort(list, smac_schedules_dlg_compare_nodes);

    while(node_list) { /* Nodes */
    
        cur_node = (smac_node_info_t*)(node_list->data);

        cur_node_tn = smac_schedules_dlg_update_nodes_add_node(node_root, cur_node,
                                                               TRUE);
        tmp_list = g_list_sort(cur_node->schedules_list, smac_schedules_dlg_compare_schedules);

        while(tmp_list) { /* Schedules this node belongs to */

            cur_schedule = (smac_schedule_info_t*)(tmp_list->data);

            cur_schedule_tn = smac_schedules_dlg_update_nodes_add_schedule(cur_node_tn, 
                                                                           cur_schedule);
            tmp_list2 = g_list_sort(g_list_first(cur_schedule->nodes_list), smac_schedules_dlg_compare_nodes);

            while(tmp_list2) { /* Set of nodes on this schedule */
	
                smac_schedules_dlg_update_nodes_add_node(cur_schedule_tn, 
                                                         (smac_node_info_t*)(tmp_list2->data),
                                                         FALSE);
                tmp_list2 = g_list_next(tmp_list2);
            }
      
            tmp_list = g_list_next(tmp_list);
        }

        node_list = g_list_next(node_list);
    }

}

void smac_schedules_dlg_update_schedules(GList *list)
{
    gchar label_text[256];
    guint foo;
	
    if (smac_schedules_dlg != NULL) {
        gtk_clist_clear(GTK_CLIST(clist));
        schedules_nb = g_list_length(smac_schedules_get_info()->schedules_list);
        schedules_ns = 0;
        g_snprintf(label_text, 256,
                   "Total: Nodes: %d   Schedules: %d   Sync Packets: %d   Packets with no known schedule: %d",
                   g_list_length(smac_schedules_get_info()->nodes_list),
                   g_list_length(smac_schedules_get_info()->schedules_list),
                   smac_schedules_get_info()->num_sync_packets,
                   g_list_length(smac_schedules_get_info()->loose_packets_list));
        gtk_label_set(GTK_LABEL(status_label), label_text);

        foo=	g_list_length(list);
        list = g_list_first(list);
        while (list)
        {
            add_to_clist((smac_schedule_info_t*)(list->data));
            list = g_list_next(list);
        }

        g_snprintf(label_text, 256,
                   "Detected %d S-MAC %s. Selected %d %s.",
                   schedules_nb, 
                   plurality(schedules_nb, "Schedule", "Schedules"),
                   schedules_ns,
                   plurality(schedules_ns, "Schedule", "Schedules"));
        gtk_label_set(GTK_LABEL(top_label), label_text);

        smac_schedules_on_unselect(NULL, NULL);
    }

    last_list = list;
}


/****************************************************************************/
/* draw function for tap listeners to keep the window up to date */
void smac_schedules_dlg_draw(void *ptr _U_)
{
    smac_schedules_dlg_update_schedules(smac_schedules_get_info()->schedules_list);
    smac_schedules_dlg_update_nodes(smac_schedules_get_info()->nodes_list);
}

/* reset function for tap listeners to clear window, if necessary */
void smac_schedules_dlg_reset(void *ptr _U_)
{
    /* Clean up memory used by schedules tap */
    smac_schedules_reset((smac_schedules_tapinfo_t*) smac_schedules_get_info()); 
}

/* init function for tap */
static void
smac_schedules_init_tap(char *dummy _U_)
{
    /*	smac_visualization_dlg_init_settings(); */

    /* Clean up memory used by schedules tap */
    smac_schedules_reset((smac_schedules_tapinfo_t*) smac_schedules_get_info()); 

    /* Register the tap listener */
	
    register_tap_listener("smac", &(smac_schedules_get_info()->dummy), NULL, 
                          smac_schedules_dlg_reset,
                          smac_schedules_packet,
                          smac_schedules_dlg_draw);

    register_tap_listener("smac_vis", &(smac_schedules_get_info()->dummy), NULL, 
                          smac_visualization_reset,
                          smac_visualization_packet,
                          smac_visualization_dlg_draw);

    /* init the Graph Analysys
       smac_visualization_data = smac_visualization_dlg_init();
    */
    smac_node_streams = NULL;
    smac_vis_tap = register_tap("smac_vis");

    /* create dialog box if necessary */
    if (smac_schedules_dlg == NULL) {
        smac_schedules_dlg_create();
    } else {
        /* There's already a dialog box; reactivate it. */
        reactivate_window(smac_schedules_dlg);
    }
	
    /* Scan for S-MAC schedules schedules (redissect all packets) */
    cf_retap_packets(&cfile);
	
    /* Tap listener will be removed and cleaned up in smac_schedules_on_destroy */
}


/****************************************************************************/
/* entry point when scheduled via the GTK menu */
void smac_schedules_launch(GtkWidget *w _U_, gpointer data _U_)
{
    smac_schedules_init_tap("");
}

/****************************************************************************/
void
register_tap_listener_smac_schedules_dlg(void)
{
    ((smac_schedules_tapinfo_t*) smac_schedules_get_info())->schedules_list = NULL; 

    register_tap_listener_cmd_arg("smac,schedules",smac_schedules_init_tap);
    register_tap_menu_item("S-MAC Schedules...", REGISTER_TAP_GROUP_GENERIC,
                           smac_schedules_launch, NULL, NULL, NULL);
}

