/* smac_visualization.h
 * Graphic Analysis addition for S-MAC
 *
 * $Id: smac_visualization.h 5197 2006-04-21 00:35:38Z smchenry $
 *
 * Copyright 2005, University of Southern California
 * By Tyler McHenry <smchenry@isi.edu>
 *
 * based on graph_analysis.h
 * Copyright 2004, Verso Technologies Inc.
 * By Alejandro Vaquero <alejandrovaquero@yahoo.com>
 *
 * based on rtp_analysis.c and io_stat
 *
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation,  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef SMAC_VISUALIZATION_H_INCLUDED
#define SMAC_VISUALIZATION_H_INCLUDED

#include <stdio.h>
#include <ctype.h>
#include <glib.h>
#include <gtk/gtk.h>
#include "gtkglobals.h"
#include <epan/address.h>
#include <epan/packet.h>
#include "smac_schedules.h"

#define SMAC_VIS_FONT_DESCRIPTOR "-adobe-helvetica-bold-r-normal--12-120-75-75-p-70-iso8859-1"
#define SMAC_VIS_ONESHOT_DURATION 500 /* useconds */
#define SMAC_VIS_SEND_HEIGHT 0.40     /* scaled */
#define SMAC_VIS_TIC_HEIGHT 0.10      /* scaled */
#define SMAC_VIS_VGRID_SPACING 10     /* seconds */
#define SMAC_VIS_EDGE_V_OFFSET 5      /* pixels */
#define SMAC_VIS_EDGE_H_OFFSET 5      /* pixels */
#define SMAC_VIS_GRID_V_OFFSET 0.5    /* scaled */
#define SMAC_VIS_GRID_H_OFFSET 1      /* pixels */
#define SMAC_VIS_LBL_CORRECTION -2    /* pixels */
#define SMAC_VIS_LBL_OFFSET 1         /* pixels */

#define IS_ONESHOT(e) ((e == event_send_oneshot_data) || (e == event_send_oneshot_sync) || \
		       (e == event_send_oneshot_rts) || (e == event_send_oneshot_cts))
#define IS_DATAPACKET(e) ((e == event_send_data_packet) || (e == event_recv_data_packet))

typedef enum {
    event_null,
    event_sleep,
    event_listen,
    event_send_oneshot_data,
    event_send_oneshot_sync,
    event_send_oneshot_rts,
    event_send_oneshot_cts,
    event_send_data_begin,
    event_send_data_packet,
    event_send_data_end,
    event_recv_data_begin,
    event_recv_data_packet,
    event_recv_data_end
} smac_node_stream_event_type_t;

typedef enum {
    exfmt_jdb_events,
    exfmt_gnuplot_events,
    exfmt_gnuplot_points
} smac_visualization_export_format_t;

typedef struct _smac_node_stream_event_t {
    smac_node_stream_event_type_t type;
    guint64 timestamp;
    char comment[40];
} smac_node_stream_event_t;

typedef struct _smac_node_stream_t {
    address addr;
    guint16 baseline;
    gboolean recv_cts;
    GList* events;
    smac_node_stream_event_t prev_sleep;
    guint32 counter;
} smac_node_stream_t;

extern GList * smac_node_streams;

extern void smac_visualization_reset();
extern gint smac_visualization_cmp_nodes(gconstpointer a,
                                         gconstpointer b);
extern gint smac_visualization_cmp_schedules(gconstpointer a,
                                             gconstpointer b);
extern int smac_visualization_packet(void *t, packet_info *pinfo, 
				     epan_dissect_t *edt, void *priv);
extern void smac_visualization_dump(FILE* ofile, smac_visualization_export_format_t format);
extern gdouble smac_visualization_type_value(smac_node_stream_event_type_t type); 
extern guint64 smac_visualization_find_prev_type_by_schedule(smac_schedule_info_t* schedule,
                                                             guint64 current_time,
                                                             smac_node_stream_event_type_t type);
extern guint64 smac_visualization_find_next_type_by_schedule(smac_schedule_info_t* schedule,
                                                             guint64 current_time,
                                                             smac_node_stream_event_type_t type);

#endif /*SMAC_VISUALIZATION_H_INCLUDED*/
