/* smac_visualization_dlg.h
 * Graphic Analysis addition for S-MAC
 *
 * $Id: smac_visualization_dlg.h 5200 2006-04-21 00:38:38Z smchenry $
 *
 * Copyright 2005, University of Southern California
 * By Tyler McHenry <smchenry@isi.edu>
 *
 * based on graph_analysis.h
 * Copyright 2004, Verso Technologies Inc.
 * By Alejandro Vaquero <alejandrovaquero@yahoo.com>
 *
 * based on rtp_analysis.c and io_stat
 *
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation,  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef SMAC_VISUALIZATION_DLG_H_INCLUDED
#define SMAC_VISUALIZATION_DLG_H_INCLUDED

#include <gdk/gdk.h>

#define SMAC_VISUALIZATION_MAGIC 0x2133

enum list_column {
    LC_DESCRIPTION = 0,
    LC_ID,
    LC_NUM_COLUMNS
};

typedef struct _smac_visualization_dlg_sources {
    gboolean show_all;
    GList* schedules;
    GList* nodes;
} smac_visualization_dlg_sources_t;

typedef struct _smac_visualization_dlg_filters {
    gboolean v_gridlines;
    gboolean h_gridlines;
    gboolean listen_cycles;
    gboolean sync_packets;
    gboolean rts_cts_packets;
    gboolean data_transfers;
    gboolean data_packets;
    gboolean use_color;
    gboolean use_text;
} smac_visualization_dlg_filters_t;

typedef struct _smac_visualization_dlg_params {
    GList* schedules_list;
    GList* nodes_list;
    GList* node_streams;
    guint64 start_time;
    guint64 duration;
    smac_visualization_dlg_sources_t sources;
    smac_visualization_dlg_filters_t filters;
} smac_visualization_dlg_params_t;

typedef struct _smac_visualization_dlg_colors {
    GdkColor lines;
    GdkColor gridlines;
    GdkColor gridlabels;
    GdkColor rts;
    GdkColor cts;
    GdkColor sync;
    GdkColor bcast;
} smac_visualization_dlg_colors_t;

typedef struct _smac_visualization_dlg {
    guint16 magic;
    GdkGC* gc;
    GtkWidget* window;
    GtkWidget* graph_area;
    GdkPixmap* backing_map;
    GtkAdjustment* zoom;
    GtkWidget* schedules;
    GtkWidget* nodes;
    smac_visualization_dlg_params_t params;  
    smac_visualization_dlg_colors_t colors;
} smac_visualization_dlg_t;


typedef struct _smac_visualization_dlg_export_dlg {
    GtkWidget* window;
    GtkWidget* cb_format;
    GtkWidget* txt_filename;
} smac_visualization_dlg_export_dlg_t;

extern smac_visualization_dlg_t* smac_visualization_dlg_open(smac_visualization_dlg_t* dlg);
extern void smac_visualization_dlg_draw(smac_visualization_dlg_t* dlg);

#endif
